import unittest
from lagmat import lagmat
import numpy as np
import numpy.testing as npt


class Test_lagmat(unittest.TestCase):

    def test1(self):
        A = np.array([1, 2, 3, 4])
        B = lagmat(A, [1, 2])
        C = np.array([[np.nan, np.nan], [1, np.nan], [2, 1], [3, 2]])
        npt.assert_array_equal(B, C)


# run
if __name__ == '__main__':
    unittest.main()



from .lagmat_func import lagmat
import numpy as np
import warnings


def diff_dth(x, order=1):
    """Differences of order d

    Parameters:
    -----------
    x : ndarray (2D)
        Time series with the oldest observation in the first row
        and the latest observation in the last row.

    order : int
        The number of differences to compute recursively
        (Default: 1 for I(1) processes.)

    Returns:
    --------
    y : ndarray (2D)
        The I(order=d) differences

    """
    y = np.array(x, copy=True, dtype=float)
    for i in range(order):
        y -= lagmat(y, lags=[1])
    return y


def diff_1st(x):
    """Difference x_t - x_{t-1} or Order of Integration I(1)

    Parameters:
    -----------
    x : ndarray (2D)
        Time series with the oldest observation in the first row
        and the latest observation in the last row.

    Returns:
    --------
    y : ndarray (2D)
        The I(1) differences x[1:, :] - x[:-1, :]

    Alternative Implementation:
    ---------------------------
        import numpy as np
        def diff_1st(x):
            y = np.empty(shape=x.shape)
            y[0, :] = np.nan
            y[1:, :] = x[1:, :] - x[:-1, :]
            return y
    """
    warnings.warn((
        "This function is for demonstration purpose only. "
        "It is equivalent to 'y = diff_dth(x, order=1)'. "
        "Please use 'diff_dth' instead."))
    return x - lagmat(x, lags=[1])



from .lagmat_func import lagmat
from .chopnan import chopnan, rmnanrow
from .diff_dth import diff_1st, diff_dth
from .diff_len import diff_len
from .cont import val_to_cont, cont_to_val, Cont
from .pct import val_to_pct, pct_to_val, Pct
from .roc import val_to_roc, roc_to_val, RoC



from .lagmat_func import lagmat
import numpy as np


def diff_len(x, lengths=None):
    """Differences for different Subperiod Lengths

    Parameters:
    -----------
    x : ndarray (2D)
        Time series with the oldest observation in the first row
        and the latest observation in the last row.

    lengths : list of int
        List/tuple with different subperiod lengths.

    Returns:
    --------
    y : ndarray (2D)
        Differences y_t = x_t - x_{t - m}
    """
    return np.tile(x, len(lengths)) - lagmat(x, lags=lengths)




from .lagmat_func import lagmat
from .chopnan import chopnan
import numpy as np
from sklearn.base import BaseEstimator, TransformerMixin


def val_to_pct(val):
    """Convert values (e.g. prices) to percentage changes, discrete returns

    Parameters:
    -----------
    x : ndarray (2D)
        Time series with the oldest observation in the first row
        and the latest observation in the last row.
        Values need to be ratio-scale, i.e. x>0

    Returns:
    --------
    ret : ndarray (2D)
        percentage changes, discrete returns.
        The first row will be nan, i.e. r[0,;]=np.nan

    Alternative:
    ------------
        def val_to_pct(val):
            y = np.empty(shape=x.shape)
            y[0, :] = np.nan
            y[1:, :] = x[1:, :] / x[:-1, :] - 1
            return y
    """
    return val / lagmat(val, lags=[1]) - 1.0


def pct_to_val(ret, initial=1):
    """Convert percentage changes, discrete returns into values or prices

    Returns:
    --------
    ret : ndarray (2D)
        percentage changes, discrete returns.

    initial: int or list/tupel/array
        Start values to compound the growth rates from

    Returns:
    --------
    x : ndarray (2D)
        Time series with the oldest observation in the first row
        and the latest observation in the last row.
    """
    z = 1.0 + ret
    z[0, :] = initial
    return np.cumprod(z, axis=0)


class Pct(BaseEstimator, TransformerMixin):
    def __init__(self, initial=None, trimnan=True):
        self.initial = initial
        self.trimnan = trimnan

    def fit(self, X, y=None):
        # store the starting values
        if self.initial is None:
            if len(X.shape) is 1:
                self.initial = X[0]
            else:
                self.initial = X[0, :]
        # done
        return self

    def transform(self, X, copy=None):
        if self.trimnan:
            return chopnan(val_to_pct(X), nchop=1)
        else:
            return val_to_pct(X)

    def inverse_transform(self, Z, initial=None, copy=None):
        # Use the requested initial value, or use the fitted initial values
        if initial is None:
            initial = self.initial
        # Multiply initial values repeatly by (1 + ret)
        if self.trimnan:
            return pct_to_val(
                np.r_[np.zeros(shape=(1, Z.shape[1])), Z],
                initial=initial)
        else:
            return pct_to_val(Z, initial=initial)



import numpy as np


def lagmat(A, lags=[], order='F'):
    """Create array with time-lagged copies of the features/variables

    Parameters
    ----------
    A : np.ndarray
        Dataset. One column for each features/variables, and one
        row for each example/observation at a certain time step.

    lags : list, tuple
        Definition what time lags the copies of A should have.

    order : str
        (Optional) Default 'F'

    Assumptions
    -----------
    * It's a time-homogenous time series (that's why there is
        no time index)
    * Features/Variables are ordered from the oldest example/observation
        (1st row) to the latest example (last row)
    * Any Missing Value treatment have been done previously.

    """
    if lags:
        # detect negative lags
        if min(lags) < 0:
            raise Exception((
                "Negative lags are not allowed. Only provided integers "
                "greater equal 0 as list/tuple elements"))

        # correct dimensions
        if len(A.shape) is 1:
            A = A.reshape(-1, 1)

        # number of colums and lags
        n_rows, n_cols = A.shape
        n_lags = len(lags)

        # allocate memory
        B = np.empty(
            shape=(n_rows, n_cols * n_lags),
            order=order,
            dtype=float)
        B[:] = np.nan

        # Copy lagged columns of A into B
        for i, l in enumerate(lags):
            # target columns of B
            j = i * n_cols
            k = j + n_cols  # (i+1) * ncols
            # number rows of A
            nl = n_rows - l
            # Copy
            B[l:, j:k] = A[:nl, :]

        return B
    else:
        return None
